// MainFrm.cpp : implementation of the CMainFrame class
//************************************************************************

#include "stdafx.h"
#include "JtiHorizontal.h"
#include "MainFrm.h"
#include "JtiHorizontalDoc.h"
#include "StarFileParser.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWndEx)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWndEx)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SIZE()
	// View
	ON_COMMAND_RANGE(ID_VIEW_APPLOOK_WIN_2000, ID_VIEW_APPLOOK_WINDOWS_7, &CMainFrame::OnApplicationLook)
	ON_UPDATE_COMMAND_UI_RANGE(ID_VIEW_APPLOOK_WIN_2000, ID_VIEW_APPLOOK_WINDOWS_7, &CMainFrame::OnUpdateApplicationLook)

	// Analysis
	ON_COMMAND(ID_ANALYSIS_RUN, OnAnalysisRun)
	ON_UPDATE_COMMAND_UI(ID_ANALYSIS_RUN, OnUpdateAnalysisRun)
	// Window
	ON_COMMAND(ID_WINDOW_CLOSEALL, OnWindowCloseall)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	Extplo = _T(".plo");

//Pointers are initialised so that it doesn't crash on win2000

	m_nRPvTSLGraph = false;
	m_nRPvTLLGraph = false;
	m_nCvTGraph	= false;
	m_nRPvCGraph = false;
	bRPvTLLOpen	= false;
	bRPvCOpen	= false;
	bRPvTSLOpen	= false;
	bCvTOpen	= false;
	m_nRPvTSLNo	= 0;
	m_nRPvTLLNo	= 0;
	m_nCvTNo	= 0;
	m_nRPvCNo	= 0;

	theApp.m_nAppLook = theApp.GetInt(_T("ApplicationLook"), ID_VIEW_APPLOOK_WINDOWS_7);
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	const CString csSrc("MainFrm::OnCreate");
	if ( CMDIFrameWndEx::OnCreate(lpCreateStruct) == -1 )
	{
		theApp.ReportError(csSrc, _T("CMDIFrameWndEx OnCreate failed"));
		return -1;
	}

	OnApplicationLook(theApp.m_nAppLook);

	EnableToolTips(true);
	SetTitleBar(_T(""));  // Set version number on titlebar
	DragAcceptFiles();

	UINT iLen = 0;
	WINDOWPLACEMENT* pWndpl;
	theApp.GetProfileBinary(_T("Settings"), _T("WindowPlacement"), (LPBYTE*)&pWndpl, &iLen);
	if (iLen > 0) SetWindowPlacement(pWndpl);
	else ShowWindow(SW_SHOWNORMAL);
	delete [] pWndpl;
	UpdateWindow();
	//OnApplicationLook(theApp.m_nAppLook);  // causes plots with cums to fail
	
	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying the CREATESTRUCT cs

	cs.style = WS_OVERLAPPED | WS_CAPTION | FWS_ADDTOTITLE
		| WS_THICKFRAME | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_MAXIMIZE;

	return CMDIFrameWndEx::PreCreateWindow(cs);
}

BOOL CMainFrame::DestroyWindow() 
{
	WINDOWPLACEMENT wndpl;
	this->GetWindowPlacement(&wndpl);
	theApp.WriteProfileBinary(_T("Settings"), _T("WindowPlacement"), (LPBYTE)&wndpl, wndpl.length);

	return CMDIFrameWndEx::DestroyWindow();
}

void CMainFrame::OnWindowCloseall() 
{
	if ( bRPvTSLOpen	) pRPvTSLChild	->DestroyWindow();
	if ( bRPvTLLOpen	) pRPvTLLChild	->DestroyWindow();
	if ( bCvTOpen		) pCvTChild		->DestroyWindow();
	if ( bRPvCOpen		) pRPvCChild	->DestroyWindow();

	FreeMemoryHandle();
}

void CMainFrame::OnSize(UINT nType, int cx, int cy)
{
	//const CString csSrc("MainFrame::OnSize");
	//ATLTRACE(csSrc + _T("\n"));
   if ( nType != SIZE_MINIMIZED )   // Force repainting if center mode is selected
	  Invalidate();
	CMDIFrameWndEx::OnSize(nType, cx, cy);
}

void CMainFrame::OnApplicationLook(UINT id)
{
	CWaitCursor wait;

	theApp.m_nAppLook = id;
	switch (id)
	{
	case ID_VIEW_APPLOOK_WIN_2000:
		CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManager));
		break;

	case ID_VIEW_APPLOOK_OFF_XP:
		CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManagerOfficeXP));
		break;

	case ID_VIEW_APPLOOK_WIN_XP:
		CMFCVisualManagerWindows::m_b3DTabsXPTheme = true;
		CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManagerWindows));
		break;

	case ID_VIEW_APPLOOK_OFF_2003:
		CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManagerOffice2003));
		CDockingManager::SetDockingMode(DT_SMART);
		break;

	case ID_VIEW_APPLOOK_VS_2005:
		CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManagerVS2005));
		CDockingManager::SetDockingMode(DT_SMART);
		break;

	case ID_VIEW_APPLOOK_VS_2008:
		CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManagerVS2008));
		CDockingManager::SetDockingMode(DT_SMART);

		break;

	case ID_VIEW_APPLOOK_WINDOWS_7:
		CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManagerWindows7));
		CDockingManager::SetDockingMode(DT_SMART);
		break;

	default:
		switch (id)
		{
		case ID_VIEW_APPLOOK_OFF_2007_BLUE:
			CMFCVisualManagerOffice2007::SetStyle(CMFCVisualManagerOffice2007::Office2007_LunaBlue);
			break;

		case ID_VIEW_APPLOOK_OFF_2007_BLACK:
			CMFCVisualManagerOffice2007::SetStyle(CMFCVisualManagerOffice2007::Office2007_ObsidianBlack);
			break;

		case ID_VIEW_APPLOOK_OFF_2007_SILVER:
			CMFCVisualManagerOffice2007::SetStyle(CMFCVisualManagerOffice2007::Office2007_Silver);
			break;

		case ID_VIEW_APPLOOK_OFF_2007_AQUA:
			CMFCVisualManagerOffice2007::SetStyle(CMFCVisualManagerOffice2007::Office2007_Aqua);
			break;
		}

		CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManagerOffice2007));
		CDockingManager::SetDockingMode(DT_SMART);
	}

	//m_wndOutput.UpdateFonts();
	RedrawWindow(NULL, NULL, RDW_ALLCHILDREN | RDW_INVALIDATE | RDW_UPDATENOW | RDW_FRAME | RDW_ERASE);

	theApp.WriteInt(_T("ApplicationLook"), id);
}

void CMainFrame::OnUpdateApplicationLook(CCmdUI* pCmdUI)
{
	pCmdUI->SetRadio(theApp.m_nAppLook == pCmdUI->m_nID);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

BOOL CMainFrame::CreateLabelMDIWindow(CMultiDocTemplate* childDocTemplate, CMDIChildWndEx* &childWindow, CString title)
{
	POSITION pos = childDocTemplate->GetFirstDocPosition();
	if(pos)
	{
		if (childWindow)
		{
			childWindow->MDIActivate();
			childWindow->ShowWindow(SW_SHOWMAXIMIZED);
		}
	}
	else
	{
		childDocTemplate->OpenDocumentFile(NULL);
		if (childWindow)
		{
			childWindow = (CMDIChildWndEx*) MDIGetActive();
			childWindow->GetActiveDocument()->SetTitle(m_nSelectedItem + _T(" - ") + title);
		}
	}
	return true;
}

HGLOBAL MyGlobalAlloc(int nRecords, CString sStr)
{
	const CString csSrc("MainFrame::MyGlobalAlloc");
	HGLOBAL h = GlobalAlloc (GHND, sizeof (double) * nRecords);
	if ( !h )
	{
		CString csErr;
		csErr.Format(_T("GlobalAlloc failed %s"), sStr);
		theApp.ReportError(csSrc, csErr);
		theApp.ExitInstance();
	}
	return h;
}

LPREAL MyGlobalLock(HGLOBAL h, CString sStr)
{
	const CString csSrc("MainFrame::MyGlobalAlloc");
	LPREAL lpR = (LPREAL) GlobalLock(h);
	if ( !lpR )
	{
		CString csErr;
		csErr.Format(_T("GlobalLock failed %s"), sStr);
		theApp.ReportError(csSrc, csErr);
		theApp.ExitInstance();
	}
	return lpR;
}

void CMainFrame::OnRPvTSLPlot() 
{
	const CString csSrc("MainFrame::OnRPvTSLPlot");

	// initialize simulated data
	LPREAL lpPrdnX1, lpPrdnY1;
	int    i;

	mapPlot = 1;
	ForFilePlotOperation();
		
	if ( m_nRPvTSLNo < 2 )
	{
		theApp.ReportError(csSrc, _T("Rate vs. Time (semi-log) plot cannot be generated....\nInsufficient number of points."));
		return;
	}

	bRPvTSLOpen	= CreateLabelMDIWindow(theApp.pRPTSLDocTemplate	, pRPvTSLChild	, GetMDITitle(theApp.CT_RPvTSL));
	if (!bRPvTSLOpen) theApp.ReportError(csSrc, _T("Failed to create Rate vs. Time (semi-log) view"));
}

void CMainFrame::OnRPvTLLPlot()
{
	const CString csSrc("MainFrame::OnRPvTLLPlot");

	// initialize simulated data
	LPREAL lpPrdnX1, lpPrdnY1;
	int    i;

	mapPlot = 2;
	ForFilePlotOperation();

	if ( m_nRPvTLLNo < 2 )
	{
		theApp.ReportError(csSrc, _T("Rate vs. Time (log-log) plot cannot be generated....\nInsufficient number of points."));
		return;
	}

	bRPvTLLOpen	= CreateLabelMDIWindow(theApp.pRPvTLLDocTemplate, pRPvTLLChild	, GetMDITitle(theApp.CT_RPvTLL));
	if (!bRPvTLLOpen) theApp.ReportError(csSrc, _T("Failed to create Rate vs. Time (log-log) view"));
	return;
}

void CMainFrame::OnCvTPlot()
{
	const CString csSrc("MainFrame::OnCvTPlot");

	// initialize simulated data
	LPREAL lpPrdnX1, lpPrdnY1;
	int    i;

	mapPlot = 3;
	ForFilePlotOperation();

	if ( m_nCvTNo < 2 )
	{
		theApp.ReportError(csSrc, _T("Cumulative Production vs. Time plot cannot be generated....\nInsufficient number of points."));
		return;
	}

	bCvTOpen		= CreateLabelMDIWindow(theApp.pCvTDocTemplate	, pCvTChild		, GetMDITitle(theApp.CT_CvT));
	if (!bCvTOpen) theApp.ReportError(csSrc, _T("Failed to create Cumulative Production vs. Time view."));
	return;
}

void CMainFrame::OnRPvCPlot()
{
	const CString csSrc("MainFrame::OnRPvCPlot");
	LPREAL lpPrdnX1, lpPrdnY1;
	int    i;

	mapPlot = 4;
	ForFilePlotOperation();

	if ( m_nRPvCNo < 2 )
	{
		theApp.ReportError(csSrc, _T("Rate vs. Cumulative plot cannot be generated....\nInsufficient number of points."));
		return;
	}

	bRPvCOpen		= CreateLabelMDIWindow(theApp.pRPvCDocTemplate	, pRPvCChild	, GetMDITitle(theApp.CT_RPvC));
	if (!bRPvCOpen) theApp.ReportError(csSrc, _T("Failed to create Rate vs. Cumulative view."));
	return;
}

int CMainFrame::CountLinesInFile(FILE *fpNoLn)
{
	// this will count the lines no matter what the length of the line
	char *ptSrc;
	const int nchar = 80;
	char tempSrc[nchar];
	int n_lines = 0;
	int n_chars = 0;

	while ( fgets(tempSrc, nchar, fpNoLn) != NULL )
	{
		n_chars = 0;
		ptSrc = tempSrc;
		while ( *ptSrc != '\n' && n_chars < nchar )
		{
			ptSrc++;
			n_chars++;
		}

		if (*ptSrc == '\n')	n_lines = n_lines + 1;
	}

	fclose(fpNoLn);
	return n_lines;
}

void CMainFrame::OnAnalysisRun()
{
	const CString csSrc("MainFrame::OnAnalysisRun");
	OnWindowCloseall();

	FreeMemoryHandle();


	m_nSelectedItem = "Bakken Horizontal";

	OnRPvTLLPlot();
//	if (theApp.m_nAppLook == ID_VIEW_APPLOOK_WINDOWS_7) MDITile(MDITILE_VERTICAL);  // Kludge to get Windows 7 to work
	OnCvTPlot();
//	if (theApp.m_nAppLook == ID_VIEW_APPLOOK_WINDOWS_7) MDITile(MDITILE_VERTICAL);  // Kludge to get Windows 7 to work
	OnRPvCPlot();
//	if (theApp.m_nAppLook == ID_VIEW_APPLOOK_WINDOWS_7) MDITile(MDITILE_VERTICAL);  // Kludge to get Windows 7 to work
	OnRPvTSLPlot();

	MDITile(MDITILE_VERTICAL);
}

CString GetErrorString(DWORD dwErr)
{
	CString ErrorMsg;

	LPTSTR szErrMsg;
	if ( FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER, NULL, dwErr, 0, (LPTSTR)&szErrMsg, 0, NULL) )
	{
		// remove new line that szErrMsg have in them.
		TCHAR * p = _tcschr(szErrMsg, _T('\r'));
		if ( p != NULL ) *p = _T('\0');
		p = _tcschr(szErrMsg, _T('\n'));
		if ( p != NULL ) *p = _T('\0');
		ErrorMsg = szErrMsg;
	}
	else
	{ // failed
		ErrorMsg.Format(_T("Unknown error %d"), dwErr);
	}
	return ErrorMsg;
}

void CMainFrame::OnUpdateAnalysisRun(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(true);
	SetTitleBar(m_nSelectedItem);
}

void CMainFrame::FreeMemoryHandle()
{
	if ( hX1			) GlobalFree(hX1			);
	if ( hY1			) GlobalFree(hY1			);
	if ( hRPvCX1		) GlobalFree(hRPvCX1		);
	if ( hRPvCY1		) GlobalFree(hRPvCY1		);
	if ( hCvTX1			) GlobalFree(hCvTX1			);
	if ( hCvTY1			) GlobalFree(hCvTY1			);
	if ( hRPvTLLX1		) GlobalFree(hRPvTLLX1		);
	if ( hRPvTLLY1		) GlobalFree(hRPvTLLY1		);
}

void CMainFrame::ForFilePlotOperation()
{
	const CString csSrc("MainFrame::ForFilePlotOperation");
	FILE *fpNoLn;
	CString tFileName;

	tFileName = m_nSelectedItem;
	tFileName = tFileName.Trim() + Extplo;
	_tfopen_s(&fpNoLn, tFileName, _T("rt"));

	if ( fpNoLn == NULL )
	{
		ATLTRACE (csSrc + _T(": Cannot open the file\n%s\n"), tFileName);
		return;
	}

	LPREAL lpX, lpY;

	if ( mapPlot == 1 ) // ---> Production Forecast - Rate Vs Time (SemiLog)
	{
		m_nRPvTSLNo = CountLinesInFile(fpNoLn);
		if ( m_nRPvTSLNo < 2 )
			return;

		hX1 = MyGlobalAlloc(m_nRPvTSLNo, csSrc + _T(" hX1"));
		hY1 = MyGlobalAlloc(m_nRPvTSLNo, csSrc + _T(" hY1"));
// get pointers to data arrays
		lpX = MyGlobalLock(hX1, csSrc + _T(" hX1"));
		lpY = MyGlobalLock(hY1, csSrc + _T(" hY1"));
	}
	else if ( mapPlot == 2 ) // ---> Production Forecast - Rate Vs Time (LogLog)
	{
		m_nRPvTLLNo = CountLinesInFile(fpNoLn);
		if ( m_nRPvTLLNo < 2 )
			return;

		hRPvTLLX1 = MyGlobalAlloc(m_nRPvTLLNo, csSrc + _T(" hRPvTLLX1"));
		hRPvTLLY1 = MyGlobalAlloc(m_nRPvTLLNo, csSrc + _T(" hRPvTLLY1"));
		// get pointers to data arrays
		lpX = MyGlobalLock(hRPvTLLX1, csSrc + _T(" hRPvTLLX1"));
		lpY = MyGlobalLock(hRPvTLLY1, csSrc + _T(" hRPvTLLY1"));
	}
	else if ( mapPlot == 3 ) // ---> Production Forecast - Cumulative Production Vs Time
	{
		m_nCvTNo = CountLinesInFile(fpNoLn);
		if ( m_nCvTNo < 2 )
			return;

		hCvTX1 = MyGlobalAlloc(m_nCvTNo, csSrc + _T(" hCvTX1"));
		hCvTY1 = MyGlobalAlloc(m_nCvTNo, csSrc + _T(" hCvTY1"));
		// get pointers to data arrays
		lpX = MyGlobalLock(hCvTX1, csSrc + _T(" hCvTX1"));
		lpY = MyGlobalLock(hCvTY1, csSrc + _T(" hCvTY1"));
	}
	else if ( mapPlot == 4 ) // ---> Production Forecast - Rate Vs Cumulative Production
	{
		m_nRPvCNo = CountLinesInFile(fpNoLn);
		if ( m_nRPvCNo < 2 )
			return;

		hRPvCX1 = MyGlobalAlloc(m_nRPvCNo, csSrc + _T(" hRPvCX1"));
		hRPvCY1 = MyGlobalAlloc(m_nRPvCNo, csSrc + _T(" hRPvCY1"));
// get pointers to data arrays
		lpX = MyGlobalLock(hRPvCX1, csSrc + _T(" hRPvCX1"));
		lpY = MyGlobalLock(hRPvCY1, csSrc + _T(" hRPvCY1"));
	}
	else
	{
		ATLTRACE (csSrc + _T(": Invalid mapplot number %d\n"), mapPlot);
	}

	// initialize simulated data 
	FILE *fpsource;
	char Source[80];
	float x, y;

	_tfopen_s(&fpsource, tFileName, _T("rt"));
	if ( fpsource == NULL )
	{
		ATLTRACE (csSrc + _T(": Cannot open the file\n%s\n"), tFileName);
		return;
	}
	CWaitCursor wait;

	int    i = 0;
	int line = 0;
	int nFields;
	vector<double> results;

	while ( fgets(Source, 80, fpsource) != NULL )
	{
		line++;
		nFields = StarFileParser::GetNDouble(Source, 5, line, tFileName, results);
		if (nFields <= 0 ) break;
		wait.Restore();

		// create x and y data to be plotted
		if ( (mapPlot == 1) || (mapPlot == 2) ) // ---> Production Forecast - Rate Vs Time (mapPlot = 1 Semilog; mapPlot = 2 LogLog)
		{
			m_nRPvTSLNo = line;  // 9.0 to try to limit plot data to that read
			m_nRPvTLLNo = line;  // 9.0 to try to limit plot data to that read
			x = (float)results[0];
			lpX [i] = x;

			y = (float)results[1];
			lpY[i] = y;
		}
		else if ( mapPlot == 3 ) // ---> Production Forecast - Cumulative Production Vs Time
		{
			m_nCvTNo = line;  // 9.0 to try to limit plot data to that read
			lpX [i] = (float)results[4] / 365.25; // Shift cums to actual time

			y = (float)results[2];
			lpY[i] = y;
		}
		else if ( mapPlot == 4 ) // --> Production Forecast - Rate Vs Cumulative Production
		{
			m_nRPvCNo = line;  // 9.0 to try to limit plot data to that read
			x = (float)results[2];
			lpX [i] = x;

			y = (float)results[1];
			lpY[i] = y;
		}
		else
		{
			ATLTRACE (csSrc + _T(": Invalid mapplot number %d\n"), mapPlot);
		}

		i++;
		results.clear();
	}
	fclose(fpsource);
}

void CMainFrame::SetTitleBar(CString sCasename)
{
	CString sb;

	sb.Format(_T("%s %s"), _T("TestBed"), _T("v0.2"));

	if ( sCasename != _T("") )
		sb.Append(_T(" - ") + sCasename);

	SetWindowText(sb);
}

CString CMainFrame::PlotTitle()
{
	return CString("Production Forecast");
}

CString CMainFrame::GetRateAxisLabelcs()
{
		return CString("Oil Rate (stb/d)");
}

CString CMainFrame::GetCumAxisLabelcs()
{
		return CString("Cumulative Production (Mstb)");
}

CString CMainFrame::GetTimeAxisLabelcs()
{
	return CString("Time (Years Elapsed)");
}

CString CMainFrame::GetMDITitle(int iCT)
{
	if (iCT == theApp.CT_RPvC	) return CString("Rate vs. Cumulative");
	if (iCT == theApp.CT_RPvTLL	) return CString("Rate vs. Time (log-log)");
	if (iCT == theApp.CT_RPvTSL	) return CString("Rate vs. Time (log-log)");
	if (iCT == theApp.CT_CvT	) return CString("Cumulative Production vs. Time");
	return _T("");
}
