// TeeFilesView.cpp : implementation file
//

#include "stdafx.h"
#include "TeeFilesView.h"
#include "MainFrm.h"
#include "TeeFilesView.h"
#include <math.h>

#include "aspect.h"
#include "axes.h"
#include "axistitle.h"
#include "brush.h"
#include "canvas.h"
#include "chartFont.h"
#include "legend.h"
#include "lineseries.h"
#include "marks.h"
#include "panel.h"
#include "pen.h"
#include "pointer.h"
#include "pointseries.h"
#include "series.h"
#include "strings.h"
#include "TeeChartDefines.h"
#include "teeFont.h"
#include "titles.h"

// CTeeFilesView

IMPLEMENT_DYNCREATE(CTeeFilesView, CView)

CTeeFilesView::CTeeFilesView()
{
	iCT = theApp.CT_UNDEFINED;
}

CTeeFilesView::~CTeeFilesView()
{
}

BEGIN_MESSAGE_MAP(CTeeFilesView, CView)
	//{{AFX_MSG_MAP(CTeeFilesView)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Manually add the EventSink map for the CView
// This can be generated for copying, by adding a TChart
// to a dialog box and auto-generating the map there.
// See also CTeeViewView.h
BEGIN_EVENTSINK_MAP(CTeeFilesView, CView)
	ON_EVENT(CTeeFilesView, IDC_TCHART1, 1  /* OnAfterDraw */, OnAfterDrawTChart, VTS_NONE)

END_EVENTSINK_MAP()

void CTeeFilesView::DoDataExchange(CDataExchange* pDX)
{
	CView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPrintingView)
	DDX_Control(pDX, IDC_TCHART1, m_Chart1);
	//}}AFX_DATA_MAP
}

BOOL CTeeFilesView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

void CTeeFilesView::ScaleAxisT (CAxis ax)
{
	ax.GetGridPen().SetStyle(psSolid);
	ax.GetTitle().GetFont().SetSize(9);
	ax .AdjustMaxMin();
	if (ax.GetLogarithmic())
	{
		ax.SetMinorTickCount(8);
		ax.SetMinorTickLength(5);
		double dMin = ax.GetMinimum();
		double dMax = ax.GetMaximum();
		int iCee;
		if (ax.GetAutomaticMinimum())
		{
			ax.SetAutomaticMinimum(false);
			iCee = floor(log10(ax.GetMinimum()));
			dMin = pow(10.0, floor(log10(ax.GetMinimum())));
			ax.SetMinimum(pow(10.0, floor(log10(ax.GetMinimum()))));
		}
		if (ax.GetAutomaticMaximum())
		{
			ax.SetAutomaticMaximum(false);
			iCee = floor(log10(ax.GetMaximum()) + 0.99);
			dMax = pow(10.0, floor(log10(ax.GetMaximum()) + 0.99));
			ax.SetMaximum(pow(10.0, floor(log10(ax.GetMaximum()) + 0.99)));
		}
	}
	else  // See OnAfterDraw for more cartesian scaling.
	{
		double dMin = 0.0;
		ax.SetAutomaticMinimum(false);
		ax.SetMinimum(dMin);
	}
}

BOOL CTeeFilesView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	if (!CWnd::Create(lpszClassName, _T(""), dwStyle, rect, pParentWnd, nID, pContext))
		return false;

	if (!m_Chart1.Create(lpszClassName, _T(""), dwStyle, rect, this, IDC_TCHART1, pContext))
		return false;

	return true;
}

void CTeeFilesView::OnInitialUpdate()
{
	const CString csSrc("TeeFilesView::OnInitialUpdate");
	CView::OnInitialUpdate();
	CMainFrame*     pFrame = (CMainFrame *) AfxGetMainWnd();
	CString csXaxisLabel, csYaxisLabel, csPlotTitle;
	CString csPlotBase = pFrame->PlotTitle();
	BOOL bXlog = false, bYlog = false;

	if (iCT == theApp.CT_RPvTSL)
	{
		csYaxisLabel = pFrame->GetRateAxisLabelcs();
		csPlotTitle = csPlotBase + _T(" semi-log)");
		bYlog = true;
	}

	if (iCT == theApp.CT_RPvTLL)
	{
		csYaxisLabel = pFrame->GetRateAxisLabelcs();
		csPlotTitle = csPlotBase + _T(" (log-log)");
		bXlog = true;
		bYlog = true;
	}
	
	if (iCT == theApp.CT_CvT)
	{
		csYaxisLabel = pFrame->GetCumAxisLabelcs();
		csPlotTitle = csPlotBase;
	}

	if (iCT == theApp.CT_RPvC)
	{
		csXaxisLabel = pFrame->GetCumAxisLabelcs();
		csYaxisLabel = pFrame->GetRateAxisLabelcs();
		csPlotTitle = csPlotBase;
		bYlog = true;
	}
	else
		csXaxisLabel = pFrame->GetTimeAxisLabelcs();

	const int nColors = 10;
	const COLORREF cWhite = RGB(255,255,255);
	const COLORREF cBlack = RGB(0,0,0);
	
	m_Chart1.GetAspect().SetView3D(false);
	m_Chart1.GetPanel().SetColor(cWhite);
	m_Chart1.SetVisible(true);

	// Title
	COleVariant var1 = csPlotTitle;
	CTitles hd = m_Chart1.GetHeader();
	hd.GetText().Clear();
	hd.GetText().Add(*(LPCVARIANT)var1);
	CTeeFont f = hd.GetFont();
	f.SetSize(10);
	f.SetColor(cBlack);

	// SubTitle
	var1 = _T("Case: ") + pFrame->m_nSelectedItem;
	hd = m_Chart1.GetSubHeader();
	hd.GetText().Clear();
	hd.GetText().Add(*(LPCVARIANT)var1);

	f = hd.GetFont();
	f.SetSize(7);
	f.SetColor(cBlack);

	int nPoints = 0;
	if		(iCT == theApp.CT_RPvC		) nPoints = pFrame->m_nRPvCNo;
	else if (iCT == theApp.CT_RPvTSL	) nPoints = pFrame->m_nRPvTSLNo;
	else if (iCT == theApp.CT_RPvTLL	) nPoints = pFrame->m_nRPvTLLNo;
	else if (iCT == theApp.CT_CvT		) nPoints = pFrame->m_nCvTNo;
	else theApp.ReportError(csSrc, _T("Invalid plot type. Contact JTI"));


	COLORREF cBlue = RGB(0,0,255);
	CSeries s;

	LPREAL lpX, lpY;
	COleSafeArray X, Y; s.AddArray(nPoints, X, Y);
	

	// Rate/Pressure vs Cum
	if (iCT == theApp.CT_RPvC)
	{
		if (nPoints > 2 )
		{
			m_Chart1.AddSeries(scLine);
			s = m_Chart1.Series(0);
			lpX = (LPREAL) GlobalLock(pFrame->GethRPvCX1());
			lpY = (LPREAL) GlobalLock(pFrame->GethRPvCY1());
			for (int i = 0; i < nPoints; i++)
			{
				s.AddXY(lpX[i], lpY[i], _T(""), cBlue);
			}
			s.SetTitle(csPlotBase);  // Set name for forecast series
			s.SetColor(cBlue);
		}
	}

	// Rate/Pressure vs Time
	if (iCT == theApp.CT_RPvTLL || iCT == theApp.CT_RPvTSL)
	{
		if (nPoints > 2 )
		{
			m_Chart1.AddSeries(scLine);
			s = m_Chart1.Series(0);
			lpX = (LPREAL) GlobalLock(pFrame->GethRPvTLLX1());
			lpY = (LPREAL) GlobalLock(pFrame->GethRPvTLLY1());
			for (int i = 0; i < nPoints; i++)
			{
				s.AddXY(lpX[i], lpY[i], _T(""), cBlue);
			}
			s.SetTitle(csPlotBase);  // Set name for forecast series
			s.SetColor(cBlue);
		}
		CAxis ax = m_Chart1.GetAxis().GetBottom();
		ax.SetAutomaticMaximum(false);
		ax.SetMaximum(20.0);
	}

	// Cum vs Time
	if (iCT == theApp.CT_CvT)
	{
		if (nPoints > 2 )
		{
			m_Chart1.AddSeries(scLine);
			s = m_Chart1.Series(0);
			lpX = (LPREAL) GlobalLock(pFrame->GethCvTX1());
			lpY = (LPREAL) GlobalLock(pFrame->GethCvTY1());
			for (int i = 0; i < nPoints; i++)
			{
				s.AddXY(lpX[i], lpY[i], _T(""), cBlue);
			}
			s.SetTitle(csPlotBase);  // Set name for forecast series
			s.SetColor(cBlue);
		}
		CAxis ax = m_Chart1.GetAxis().GetBottom();
		ax.SetAutomaticMaximum(false);
		ax.SetMaximum(20.0);
	}

	// Axis
	CAxis ax = m_Chart1.GetAxis().GetBottom();
	ax.GetTitle().SetCaption(csXaxisLabel);
	ax.SetLogarithmic(bXlog);
	ax.GetMinorGrid().SetVisible(bXlog);
	ScaleAxisT(ax);

	ax = m_Chart1.GetAxis().GetLeft();
	ax.GetTitle().SetCaption(csYaxisLabel);
	ax.SetLogarithmic(bYlog);
	ax.GetMinorGrid().SetVisible(bYlog);
	ScaleAxisT(ax);

	CLegend lg = m_Chart1.GetLegend();
	lg.SetVisible(false);
}

// CTeeFilesView drawing

void CTeeFilesView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	// TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CTeeFilesView printing

////////////////////////////////////////////////////////////////////////////
// CTeeFilesView diagnostics

#ifdef _DEBUG
void CTeeFilesView::AssertValid() const
{
	CView::AssertValid();
}

void CTeeFilesView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CJtiHorizontalDoc* CTeeFilesView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CJtiHorizontalDoc)));
	return (CJtiHorizontalDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTeeFilesView message handlers

void CTeeFilesView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	 //routine sizes the Chart to the Client window
	 if (m_Chart1.m_hWnd)
	 {
		  m_Chart1.SetWidth(cx);
		  m_Chart1.SetHeight(cy);
	 }	
}

//TChart Events

void CTeeFilesView::ScaleCartesian(CAxis ax)
{
	const CString csSrc("TeeFilesView::ScaleCartesian");
	if (!ax.GetLogarithmic())
	{
		double dMin = ax.GetMinimum();
		double dMax = ax.GetMaximum();
		double dIncr = ax.CalcIncrement();
		ATLTRACE(csSrc + _T(": plot, min, max, inc = %d %f %f %f\n"), iCT, dMin, dMax, dIncr);
		if(ax.GetAutomaticMaximum())
		{
			dMax = (floor((dMax - dMin)/dIncr) + 1) * dIncr + dMin;
			ax.SetAutomaticMaximum(false);
			ax.SetMaximum(dMax);
			ax.SetIncrement(dIncr);
			m_Chart1.RedrawWindow();
		}
	}
}

void CTeeFilesView::OnAfterDrawTChart()
{
	CAxes axes = m_Chart1.GetAxis();
	ScaleCartesian(axes.GetBottom());
	ScaleCartesian(axes.GetLeft());
}
